/*-----------------------------------------------------------------------------
-----------------------------------------------------------------------------*/
#include <nds.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <nds/arm9/console.h> //basic print funcionality

#include "kernel.h"
#include "kernel_id.h"

uint32 kernel_int_stk[64];
VP kernel_int_sp = (VP)(kernel_int_stk + 64);

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int main(void) {
  REG_IME = 0;

  videoSetMode(0);	//not using the main screen
  videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);	//sub bg 0 will be used to print text
  vramSetBankC(VRAM_C_SUB_BG);

  SUB_BG0_CR = BG_MAP_BASE(31);

  BG_PALETTE_SUB[255] = RGB15(31,31,31);	//by default font will be rendered with color 255

  //consoleInit() is a lot more flexible but this gets you up and running quick
  consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31),
					 (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

  iprintf("\nHOS-v4 (uITRON)\n");

  // ------------------------------------------------------------- HOS $B5/F0(B ---
  sta_hos();

  for (;;) ;

  return 0;
}


/* ------------------------------------------------------------------------- */
/*  HOS $B=i4|2=(B (system.cfg $B$G;XDj(B)                                           */
/* ------------------------------------------------------------------------- */
void Initialize(VP_INT exinf)
{
  act_tsk(TSK_ID_TASK1);
  act_tsk(TSK_ID_TASK2);

  REG_IME = 1;

  wai_sem(SEM_ID_BGTEXT);
  iprintf("init OK\n");
  sig_sem(SEM_ID_BGTEXT);
}


/* ------------------------------------------------------------------------- */
/*  $B%?%9%/(B 1                                                                 */
/* ------------------------------------------------------------------------- */
void Task1(VP_INT exinf)
{
  touchPosition touchXY;

  wai_sem(SEM_ID_BGTEXT);
  iprintf("Task1 Launched\n");	// \x1b[15;0H
  sig_sem(SEM_ID_BGTEXT);

  while (1) {
	dly_tsk(100);

	touchXY = touchReadXY();

	wai_sem(SEM_ID_BGTEXT);
	iprintf("\x1b[9;0HTask1 checks touch screen.");
	iprintf("\x1b[10;0HTouch x = %04X, %04X\n", touchXY.x, touchXY.px);
	iprintf("Touch y = %04X, %04X\n", touchXY.y, touchXY.py);
	sig_sem(SEM_ID_BGTEXT);
  }
}

/* ------------------------------------------------------------------------- */
/*  $B%?%9%/(B 2                                                                 */
/* ------------------------------------------------------------------------- */
void Task2(VP_INT exinf)
{
  int16 wCnt = 0;

  wai_sem(SEM_ID_BGTEXT);
  iprintf("Task2 Launched");	// \x1b[16;0H
  sig_sem(SEM_ID_BGTEXT);

  while (1) {
	dly_tsk(1000);

	wai_sem(SEM_ID_BGTEXT);
	iprintf("\x1b[14;0HTask2 counts %d.", wCnt);
	sig_sem(SEM_ID_BGTEXT);

	wCnt ++;
  }
}
